#include <Arduino.h>
#include <TFT_eSPI.h>
#include <SPI.h>
#include <faces.h>
#include <Wire.h>
#include <ESP32Servo.h>

uint32_t delayMS;

const int trigPin = 13; // Trig pin for HC-SR04
const int echoPin = 12; // Echo pin for HC-SR04
const int servoPin = 32; // Pin for the servo motor
int touchPin = 15; // Touch pin for the touch sensor
int touchVal = 0; // Variable to store the touch state
bool isExcited = false; // Variable to track if the excited face is shown
int lastTouchVal = -1; // Variable to store the last touch state
unsigned long lastTouched = 0;  // the last time the touch sensor was pressed or released
unsigned long debounceDelay = 50; // the delay time to avoid bouncing (if the touch sensor is pressed or released multiple times in quick succession)
unsigned long lastDistanceCheckTime = 0;
const unsigned long distanceCheckInterval = 100; // Interval to check the distance

long duration;
int distance;
Servo myservo;

int measureHCSR04Distance();
void adjustServo();

void setup()
{
  Serial.begin(115200); // Used for messages

  pinMode(touchPin, INPUT);
  pinMode(trigPin, OUTPUT); // Sets the trigPin as an Output
  pinMode(echoPin, INPUT);  // Sets the echoPin as an Input

  myservo.attach(servoPin); // Attach the servo to the pin
  tft.init();
  tft.setRotation(2);

  myservo.write(0); // Start servo at middle position
}

void loop()
{
  int reading = digitalRead(touchPin);

  if (reading != lastTouchVal) {
    lastTouched = millis();
  }

  if ((millis() - lastTouched) > debounceDelay) {
    if (reading != touchVal) {
      touchVal = reading;

      if (touchVal == HIGH) {
        drawHappyFace();
        Serial.println("Touch detected");
      } else {
        drawSadFace();
        Serial.println("Touch released");
      }
    }
  }
  lastTouchVal = reading;

  // Check distance at regular intervals
  if (millis() - lastDistanceCheckTime >= distanceCheckInterval) {
    distance = measureHCSR04Distance();
    Serial.print("Distance: ");
    Serial.println(distance);
    
    if (distance < 10 && !isExcited) {
      drawExcitedFace();
      Serial.println("Excited face shown");
      adjustServo();
      isExcited = true;
    } else if (distance >= 10 && isExcited) {
      drawSadFace();
      isExcited = false;
    }

    lastDistanceCheckTime = millis();
  }
}

int measureHCSR04Distance() {
  digitalWrite(trigPin, LOW); // Clears the trigPin
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH); // Sets the trigPin on HIGH state for 10 micro seconds
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW); // Clears the trigPin

  duration = pulseIn(echoPin, HIGH); 
  int distance = duration * 0.034 / 2;
  return distance;
}

void adjustServo() {
  for (int i = 0; i < 2; i++) {
    myservo.write(0);
    delay(50);
    myservo.write(45);
    delay(50);
    myservo.write(0);
    delay(50);
    Serial.println("Servo moved to 90 degrees");
  }
}


